use std::collections::HashMap;

use crate::types::response::{
    DictionaryResponse, DictionaryResponseData,
    DictionaryResponseDataDicValueDic, DictionaryResponseDataDicValueUl,
};

#[derive(Debug)]
pub struct DictionaryList {
    pub next: Option<u64>,
    pub dics: Vec<DictionaryResponseDataDicValueDic>,
}

pub struct Client {
    client: reqwest::Client,
}

impl Client {
    pub fn new() -> Self {
        let client = reqwest::Client::new();
        Self { client }
    }
    pub async fn get_dictionary_list(
        &self,
        query: &str,
        start: u64,
    ) -> anyhow::Result<DictionaryList> {
        let url = "https://pictsense.com/get/dic/";
        let mut params = HashMap::new();
        let start = start.to_string();
        let now_unixtime = chrono::Utc::now().timestamp().to_string();
        params.insert("q", query);
        params.insert("s", &start);
        params.insert("t", "0");
        params.insert("at", &now_unixtime);

        let res = self.client.post(url).form(&params).send().await?;
        let body = res.text().await?;
        let dics = match serde_xml_rs::from_str::<DictionaryResponse>(&body) {
            Ok(dic) => dic,
            Err(e) => return Err(anyhow::Error::new(e).context(body)),
        };

        let dic = match dics {
            DictionaryResponse::Data(data) => match data {
                DictionaryResponseData::Dics(dic) => dic,
            },
        };

        let dic_value = "<ul>".to_string() + dic.value.as_str() + "</ul>";

        // ライブラリのバグを運用でカバー！！！！✨
        // SEE: https://github.com/RReverser/serde-xml-rs/issues/66
        let delete_span_regex = regex::Regex::new(r"<span>\([0-9]*\)<\/span>").unwrap();
        let dic_value = delete_span_regex.replace_all(&dic_value, "").to_string();

        let dic_value: DictionaryResponseDataDicValueUl = serde_xml_rs::from_str(&dic_value)?;

        Ok(DictionaryList {
            next: dic.next,
            dics: dic_value.li,
        })
    }
    pub async fn get_all_dictionary_list(
        &self,
        query: &str,
    ) -> anyhow::Result<Vec<DictionaryResponseDataDicValueDic>> {
        let mut start = 0;
        let mut dics = Vec::new();
        loop {
            println!("start: {}", start);
            let dic = self.get_dictionary_list(query, start).await?;
            dics.extend(dic.dics);
            if dic.next.is_none() {
                break;
            };
            start = dic.next.unwrap();
            tokio::time::sleep(std::time::Duration::from_millis(100)).await;
        }
        Ok(dics)
    }
    pub async fn get_dictionary_values(&self, id: u64) -> anyhow::Result<Vec<String>> {
        let url = format!("https://pictsense.com/dic/n{}", id);

        let res = self.client.get(&url).send().await?;
        let body = res.text().await?;
        let document = scraper::Html::parse_document(&body);

        // <ul id="words">
        let selector = scraper::Selector::parse("ul#words").unwrap();
        let words_list = document.select(&selector).next().unwrap();

        let mut words = Vec::new();

        for element in words_list.select(&scraper::Selector::parse("li").unwrap()) {
            words.push(element.inner_html());
        }

        Ok(words)
    }
}
